library(glmnet)
library(hydroGOF)
library(data.table)  
library(plyr)
#hydroGOF is used to calculate RMSEs a little bit more easily.
#Data Table helps picking out values of lambda
#Discards models with Less than 5 or more than 100 nonzero coefficients.

#Import Data
RegularList <- read.csv("RegularListScaled.csv", header = TRUE, stringsAsFactors = FALSE)
PlayoffsList <- read.csv("PlayoffsListScaled.csv", header = TRUE, stringsAsFactors = FALSE)
PlayoffsList2 <- read.csv("PlayoffsListScaled2.csv", header = TRUE, stringsAsFactors = FALSE)


RegularPredictor <- data.frame(RegularList[,4:1015])
PlayoffsPredictor <- data.frame(PlayoffsList[,4:1015])
PlayoffsPredictor2 <- data.frame(PlayoffsList2[,4:1015])


RegularDataResponse <- data.frame(RegularList["WinsResponse"])
PlayoffsDataResponse <- data.frame(PlayoffsList["WinsResponse"])
PlayoffsDataResponse2 <- data.frame(PlayoffsList2["WinsResponse"])



#This code can take several hours to run (6-12) depending on the speed of the computer. It can be run on a supercomputer relatively quickely if time is an issue.

# To pick parameters for the elastic-net models we use cross-fold validation. To do this, we gather all of the sample data and divide it into different groups.
# For example, 9 of the 10 groups may be used to fit the model, and the 10th group may be used to calculate the error of the model. Then a different combination of 9 are 
# used and this process is repeated. This is repeated for all of the combinations and the "real" error can be estimated by averaging the errors of different combinations.

# A foldid is a structure that randomly assigns the data into equally sized groups. A foldid is generated and then cross-fold validation is applied to a model with alpha = .01.
# The error of the model is found with 500 different combinations of lambda, and then an error score is generated for the best value of lambda. This tells us how good of a fit
# alpha = .01 is when it is matched with its optimal strength of penalty. This is repeated for all other values of alpha, and the error-minimizing value of alpha is chosen.
# Then this process is repeated with a new foldid, and the best alpha is chosen again. This occurs 100 times and the average best alpha is choses as the alpha the model should use.
RegularElasticModels <- list()
RegularAlphas <- vector(mode ="numeric", length = 101) 
RegularAlphaSum = 0
for(j in 1:100)
{
  foldid<- sample(1:10,size=length(as.vector(as.matrix(RegularDataResponse))),replace = TRUE)
  for(i in 0:100)
  {
    RegularElasticModels[[i+1]] <- cv.glmnet(x=data.matrix(RegularPredictor),y= as.vector(as.matrix(RegularDataResponse)), alpha = i/100, nlambda = 500, foldid=foldid) #Fits model
    RegularAlphas[i+1] <- rmse(predict(RegularElasticModels[[i+1]], data.matrix(RegularPredictor), RegularElasticModels[[i+1]]$lambda.min),RegularDataResponse) #Finds error
    if(nnzero(coef(RegularElasticModels[[i+1]])) > 100) {RegularAlphas[i+1] <- 1000} #Solutions that fit too many coefficients are set to have a large error so they aren't chosen. (We want a sparse solution!)
    if(nnzero(coef(RegularElasticModels[[i+1]])) < 5) {RegularAlphas[i+1] <- 1000} #Solutons containing too few coefficients are also set to have a large error (We still want meaningful solutions!)
  }
  #Since RegularAlphas[1] contains alpha =0 and so forth, the -1 ensures the correcet value of alpha is recorded.
  RegularAlpha <- which.min(RegularAlphas)-1
  RegularAlphaSum <- RegularAlphaSum + RegularAlpha
}

#The code below just repeats the process but looks at playoff or playoff2 data instead.

PlayoffsElasticModels <- list()
PlayoffsAlphas <- vector(mode ="numeric", length = 101)
PlayoffsAlphaSum = 0
for(j in 1:100)
{
  #I was receiving errors assigning the foldid "invalid" so I had to hardcode its length here.
  #Strangely, this was only an error with the Playoffs Data, so only it was changed.
  #However, if similar problems later occur I imagine a similar change can be made.
  foldid= sample(rep(seq(10),length=80))
  for(i in 0:100)
  {
    PlayoffsElasticModels[[i+1]] <- cv.glmnet(x=data.matrix(PlayoffsPredictor),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha = i/100, nlambda=500, foldid = foldid)
    PlayoffsAlphas[i+1] <- rmse(predict(PlayoffsElasticModels[[i+1]], data.matrix(PlayoffsPredictor), PlayoffsElasticModels[[i+1]]$lambda.min),PlayoffsDataResponse)
    if(nnzero(coef(PlayoffsElasticModels[[i+1]])) > 100) {PlayoffsAlphas[i+1] <- 1000}
    if(nnzero(coef(PlayoffsElasticModels[[i+1]])) < 5) {PlayoffsAlphas[i+1] <- 1000}
  }
  PlayoffsAlpha <- which.min(PlayoffsAlphas)-1
  PlayoffsAlphaSum <- PlayoffsAlphaSum + PlayoffsAlpha
}



PlayoffsElasticModels2 <- list()
PlayoffsAlphas2 <- vector(mode ="numeric", length = 101)
PlayoffsAlphaSum2 = 0
for(j in 1:100)
{
  foldid= sample(rep(seq(10),length=80))
  for(i in 0:100)
  {
    PlayoffsElasticModels2[[i+1]] <- cv.glmnet(x=data.matrix(PlayoffsPredictor2),y= as.vector(as.matrix(PlayoffsDataResponse2)), alpha = i/100, nlambda = 500, foldid=foldid)
    PlayoffsAlphas2[i+1] <- rmse(predict(PlayoffsElasticModels2[[i+1]], data.matrix(PlayoffsPredictor2), PlayoffsElasticModels2[[i+1]]$lambda.min),PlayoffsDataResponse2)
    if(nnzero(coef(PlayoffsElasticModels2[[i+1]])) > 100) {PlayoffsAlphas2[i+1] <- 1000}
    if(nnzero(coef(PlayoffsElasticModels2[[i+1]])) < 5) {PlayoffsAlphas2[i+1] <- 1000}
  }
  PlayoffsAlpha2 <- which.min(PlayoffsAlphas2)-1
  PlayoffsAlphaSum2 <- PlayoffsAlphaSum2 + PlayoffsAlpha2
}


#This process was repeated 100 time, so we divide the sums by 100.
#The index of the best alpha was also used, so since alpha[1] would represent alpha =.01, we divided the sumbs by 100 again for a total of 10,000.

RegularAlphaSum/10000
PlayoffsAlphaSum/10000
PlayoffsAlphaSum2/10000




