library(glmnet)
library(hydroGOF)
library(data.table)  
library(plyr)
scipen = 9000
#Scipen disables scientific notation in outputs.
#hydroGOF is used to calculate RMSEs a little bit more easily.
#Data Table & Plyr used in a function to calculate best lambda.


#OOS Prefix denotes Out Of Sample Data
#2 Postfix denotes using regular season data to make post season predictions


#Import Data
RegularList <- read.csv("RegularListScaled.csv", header = TRUE, stringsAsFactors = FALSE)
PlayoffsList <- read.csv("PlayoffsListScaled.csv", header = TRUE, stringsAsFactors = FALSE)
PlayoffsList2 <- read.csv("PlayoffsListScaled2.csv", header = TRUE, stringsAsFactors = FALSE)
OOSRegularList  <- read.csv("OOSRegularListScaled.csv", header = TRUE, stringsAsFactors = FALSE)
OOSPlayoffsList  <- read.csv("OOSPlayoffsListScaled.csv", header = TRUE, stringsAsFactors = FALSE)
OOSPlayoffsList2 <- read.csv("OOSPlayoffsListScaled2.csv", header = TRUE, stringsAsFactors = FALSE)
RecencyList <- read.csv("RecencyListScaled.csv", header = TRUE, stringsAsFactors = FALSE)

#Note that the data set used does not include the cap space stat. This is because % Cap Used provides
#The same information and factors in inflation, so having both in might "confuse" the regression.
RegularPredictor <- data.frame(RegularList[,5:1015])
PlayoffsPredictor <- data.frame(PlayoffsList[,5:1015])
PlayoffsPredictor2 <- data.frame(PlayoffsList2[,5:1015])
OOSRegularPredictor <- data.frame(OOSRegularList[,5:1015])
OOSPlayoffsPredictor <- data.frame(OOSPlayoffsList[,5:1015])
OOSPlayoffsPredictor2 <- data.frame(OOSPlayoffsList2[,5:1015])
RecencyList <- read.csv("RecencyListScaled.csv", header = TRUE, stringsAsFactors = FALSE)


RegularDataResponse <- data.frame(RegularList["WinsResponse"])
PlayoffsDataResponse <- data.frame(PlayoffsList["WinsResponse"])
OOSRegularDataResponse <- data.frame(OOSRegularList["WinsResponse"])
OOSPlayoffsDataResponse <- data.frame(OOSPlayoffsList["WinsResponse"])



Lambdas <- function(...) {
  cv <- cv.glmnet(...)
  return(data.table(cvm=cv$cvm, lambda=cv$lambda))
}

OptimLambda <- function(k, ...) {
  # Returns optimal lambda for glmnet.
  #
  # Args:
  #   k: # times to loop through cv.glmnet.
  #   ...: Other args passed to cv.glmnet.
  #
  # Returns:
  #   Lambda associated with minimum average CV error over runs.
  #
  # Example:
  #   OptimLambda(k=100, y=y, x=x, alpha=alpha, nfolds=k)
  #
  require(parallel)
  require(data.table)
  MSEs <- data.table(rbind.fill(mclapply(seq(k), function(dummy) Lambdas(...))))
  return(MSEs[, list(mean.cvm=mean(cvm)), lambda][order(mean.cvm)][1]$lambda)
}

#Find the optimal lambda (strength of penalty) for each different model.
LassoRegularLambda <- OptimLambda(k=100,x=data.matrix(RegularPredictor),y= as.vector(as.matrix(RegularDataResponse)), alpha =1,nfolds=10)
LassoPlayoffsLambda <- OptimLambda(k=100,x=data.matrix(PlayoffsPredictor),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =1,nfolds=10)
LassoPlayoffsLambda2 <- OptimLambda(k=100,x=data.matrix(PlayoffsPredictor2),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =1,nfolds=10)


RidgeRegularLambda <- OptimLambda(k=100,x=data.matrix(RegularPredictor),y= as.vector(as.matrix(RegularDataResponse)), alpha =0,nfolds=10)
RidgePlayoffsLambda <- OptimLambda(k=100,x=data.matrix(PlayoffsPredictor),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =0,nfolds=10)
RidgePlayoffsLambda2 <- OptimLambda(k=100,x=data.matrix(PlayoffsPredictor2),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =0,nfolds=10)


ElasticRegularLambda <- OptimLambda(k=100,x=data.matrix(RegularPredictor),y= as.vector(as.matrix(RegularDataResponse)), alpha =.7805,nfolds=10)
ElasticPlayoffsLambda <- OptimLambda(k=100,x=data.matrix(PlayoffsPredictor),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =.0986,nfolds=10)
ElasticPlayoffsLambda2 <- OptimLambda(k=100,x=data.matrix(PlayoffsPredictor2),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =.025,nfolds=10)

#Alpha = 1 Lasso, Alpha = 0 Ridge, nlambda is an upperbound for the different number of lambdas to be calculated
#Get fits
LassoRegular <- glmnet(x=data.matrix(RegularPredictor),y= as.vector(as.matrix(RegularDataResponse)), alpha =1, lambda = LassoRegularLambda)
LassoPlayoffs <- glmnet(x=data.matrix(PlayoffsPredictor),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =1, lambda = LassoPlayoffsLambda)
LassoPlayoffs2 <- glmnet(x=data.matrix(PlayoffsPredictor2),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =1, lambda = LassoPlayoffsLambda2)
RidgeRegular <- glmnet(x=data.matrix(RegularPredictor),y= as.vector(as.matrix(RegularDataResponse)), alpha =0, lambda = RidgeRegularLambda)
RidgePlayoffs <- glmnet(x=data.matrix(PlayoffsPredictor),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =0, lambda = RidgePlayoffsLambda)
RidgePlayoffs2 <- glmnet(x=data.matrix(PlayoffsPredictor2),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =0, lambda = RidgePlayoffsLambda2)


#Alpha values come from the alpha selection program. We use the lambda calculated from the optimlambda function.
ElasticRegular <- glmnet(x=data.matrix(RegularPredictor),y= as.vector(as.matrix(RegularDataResponse)), alpha =.7805, lambda = ElasticRegularLambda)
ElasticPlayoffs <- glmnet(x=data.matrix(PlayoffsPredictor),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =0.0986, lambda = ElasticPlayoffsLambda)
ElasticPlayoffs2 <- glmnet(x=data.matrix(PlayoffsPredictor2),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =.025, lambda = ElasticPlayoffsLambda2)




#Lamba min selects the value of lambda that gives the lowest Cross Validation Error
#Alternatively, Lambda 1se can be used to give a lamba that has error within 1 standard deviation of the min model. This yields a simpler model.
#Since we made the models using just one value of lambda, lambda.min = lambda.1se = OptimLambda. Typically, one model will consist of many lambda values to choose from.

#Make Predictions
LassoRegularPredictions <- predict(LassoRegular, data.matrix(RegularPredictor), LassoRegular$lambda.min)
LassoPlayoffsPredictions <- predict(LassoPlayoffs, data.matrix(PlayoffsPredictor), LassoPlayoffs$lambda.min)
LassoPlayoffsPredictions2 <- predict(LassoPlayoffs2, data.matrix(PlayoffsPredictor2), LassoPlayoffs2$lambda.min)
LassoOOSRegularPredictions <- predict(LassoRegular, data.matrix(OOSRegularPredictor), LassoRegular$lambda.min)
LassoOOSPlayoffsPredictions <- predict(LassoPlayoffs, data.matrix(OOSPlayoffsPredictor), LassoPlayoffs$lambda.min)
LassoOOSPlayoffsPredictions2 <- predict(LassoPlayoffs2, data.matrix(OOSPlayoffsPredictor2), LassoPlayoffs2$lambda.min)



RidgeRegularPredictions <- predict(RidgeRegular, data.matrix(RegularPredictor), RidgeRegular$lambda.min)
RidgePlayoffsPredictions <- predict(RidgePlayoffs, data.matrix(PlayoffsPredictor), RidgePlayoffs$lambda.min)
RidgePlayoffsPredictions2 <- predict(RidgePlayoffs2, data.matrix(PlayoffsPredictor2), RidgePlayoffs2$lambda.min)
RidgeOOSRegularPredictions <- predict(RidgeRegular, data.matrix(OOSRegularPredictor), RidgeRegular$lambda.min)
RidgeOOSPlayoffsPredictions <- predict(RidgePlayoffs, data.matrix(OOSPlayoffsPredictor), RidgePlayoffs$lambda.min)
RidgeOOSPlayoffsPredictions2 <- predict(RidgePlayoffs2, data.matrix(OOSPlayoffsPredictor2), RidgePlayoffs2$lambda.min)
RidgeRecencyPredictions <- predict(RidgePlayoffs2, data.matrix(RecencyPredictor), RidgePlayoffs2$lambda.min)

ElasticRegularPredictions <- predict(ElasticRegular, data.matrix(RegularPredictor), ElasticRegular$lambda.min)
ElasticPlayoffsPredictions <- predict(ElasticPlayoffs, data.matrix(PlayoffsPredictor), ElasticPlayoffs$lambda.min)
ElasticPlayoffsPredictions2 <- predict(ElasticPlayoffs2, data.matrix(PlayoffsPredictor2), ElasticPlayoffs2$lambda.min)
ElasticOOSRegularPredictions <- predict(ElasticRegular, data.matrix(OOSRegularPredictor), ElasticRegular$lambda.min)
ElasticOOSPlayoffsPredictions <- predict(ElasticPlayoffs, data.matrix(OOSPlayoffsPredictor), ElasticPlayoffs$lambda.min)
ElasticOOSPlayoffsPredictions2 <- predict(ElasticPlayoffs2, data.matrix(OOSPlayoffsPredictor2), ElasticPlayoffs2$lambda.min)
ElasticRecencyPredictions <- predict(ElasticPlayoffs2, data.matrix(RecencyPredictor), ElasticPlayoffs2$lambda.min)




#Output Coefficients to text files.
capture.output(coef(LassoRegular, s=LassoRegular$lambda.min), file ="LassoRegularCoefficients.txt")
capture.output(coef(LassoPlayoffs, s=LassoPlayoffs$lambda.min), file ="LassoPlayoffsCoefficients.txt")
capture.output(coef(LassoPlayoffs2, s=LassoPlayoffs2$lambda.min), file ="LassoPlayoffsCoefficients2.txt")



capture.output(coef(RidgeRegular, s=RidgeRegular$lambda.min), file ="RidgeRegularCoefficients.txt")
capture.output(coef(RidgePlayoffs, s=RidgePlayoffs$lambda.min), file ="RidgePlayoffsCoefficients.txt")
capture.output(coef(RidgePlayoffs2, s=RidgePlayoffs2$lambda.min), file ="RidgePlayoffsCoefficients2.txt")

capture.output(coef(ElasticRegular, s=ElasticRegular$lambda.min), file ="ElasticRegularCoefficients.txt")
capture.output(coef(ElasticPlayoffs, s=ElasticPlayoffs$lambda.min), file ="ElasticPlayoffsCoefficients.txt")
capture.output(coef(ElasticPlayoffs2, s=ElasticPlayoffs2$lambda.min), file ="ElasticPlayoffsCoefficients2.txt")

#Output predictions to spreadsheets.
write.csv(data.frame((LassoRegularPredictions)), "LassoRegularPredictions.csv")
write.csv(data.frame((LassoPlayoffsPredictions)), "LassoPlayoffsPredictions.csv")
write.csv(data.frame((LassoPlayoffsPredictions2)), "LassoPlayoffsPredictions2.csv")
write.csv(data.frame((LassoOOSRegularPredictions)), "LassoOOSRegularPredictions.csv")
write.csv(data.frame((LassoOOSPlayoffsPredictions)), "LassoOOSPlayoffsPredictions.csv")
write.csv(data.frame((LassoOOSPlayoffsPredictions2)), "LassoOOSPlayoffsPredictions2.csv")


write.csv(data.frame((RidgeRegularPredictions)), "RidgeRegularPredictions.csv")
write.csv(data.frame((RidgePlayoffsPredictions)), "RidgePlayoffsPredictions.csv")
write.csv(data.frame((RidgePlayoffsPredictions2)), "RidgePlayoffsPredictions2.csv")
write.csv(data.frame((RidgeOOSRegularPredictions)), "RidgeOOSRegularPredictions.csv")
write.csv(data.frame((RidgeOOSPlayoffsPredictions)), "RidgeOOSPlayoffsPredictions.csv")
write.csv(data.frame((RidgeOOSPlayoffsPredictions2)), "RidgeOOSPlayoffsPredictions2.csv")
write.csv(data.frame((RidgeOOSPlayoffsPredictions2)), "RidgeOOSPlayoffsPredictions2.csv")
write.csv(data.frame((RidgeRecencyPredictions)), "RidgeRecencyPredictions.csv")

write.csv(data.frame((ElasticRegularPredictions)), "ElasticRegularPredictions.csv")
write.csv(data.frame((ElasticPlayoffsPredictions)), "ElasticPlayoffsPredictions.csv")
write.csv(data.frame((ElasticPlayoffsPredictions2)), "ElasticPlayoffsPredictions2.csv")
write.csv(data.frame((ElasticOOSRegularPredictions)), "ElasticOOSRegularPredictions.csv")
write.csv(data.frame((ElasticOOSPlayoffsPredictions)), "ElasticOOSPlayoffsPredictions.csv")
write.csv(data.frame((ElasticOOSPlayoffsPredictions2)), "ElasticOOSPlayoffsPredictions2.csv")
write.csv(data.frame((ElasticRecencyPredictions)), "ElasticRecencyPredictions.csv")





