library(glmnet)
library(hydroGOF)
library(data.table)  
library(plyr)
scipen = 9000
#Scipen disables scientific notation.
#hydroGOF is used to calculate RMSEs a little bit more easily.
#Data Table & Plyr used in a function to calculate best lambda.


#OOS Prefix denotes Out Of Sample Data
#2 Postfix denotes using regular season data to make post season predictions


#Import Data
RegularList <- read.csv("RegularListScaled.csv", header = TRUE, stringsAsFactors = FALSE)
PlayoffsList <- read.csv("PlayoffsListScaled.csv", header = TRUE, stringsAsFactors = FALSE)
PlayoffsList2 <- read.csv("PlayoffsListScaled2.csv", header = TRUE, stringsAsFactors = FALSE)
OOSRegularList  <- read.csv("OOSRegularListScaled.csv", header = TRUE, stringsAsFactors = FALSE)
OOSPlayoffsList  <- read.csv("OOSPlayoffsListScaled.csv", header = TRUE, stringsAsFactors = FALSE)
OOSPlayoffsList2 <- read.csv("OOSPlayoffsListScaled2.csv", header = TRUE, stringsAsFactors = FALSE)
RecencyList <- read.csv("RecencyListScaled.csv", header = TRUE, stringsAsFactors = FALSE)

RegularPredictor <- data.frame(RegularList[,5:1015])
PlayoffsPredictor <- data.frame(PlayoffsList[,5:1015])
PlayoffsPredictor2 <- data.frame(PlayoffsList2[,5:1015])
OOSRegularPredictor <- data.frame(OOSRegularList[,5:1015])
OOSPlayoffsPredictor <- data.frame(OOSPlayoffsList[,5:1015])
OOSPlayoffsPredictor2 <- data.frame(OOSPlayoffsList2[,5:1015])
RecencyPredictor <- data.frame(RecencyList[,5:1015])

RegularDataResponse <- data.frame(RegularList["WinsResponse"])
PlayoffsDataResponse <- data.frame(PlayoffsList["WinsResponse"])
OOSRegularDataResponse <- data.frame(OOSRegularList["WinsResponse"])
OOSPlayoffsDataResponse <- data.frame(OOSPlayoffsList["WinsResponse"])



Lambdas <- function(...) {
  cv <- cv.glmnet(...)
  return(data.table(cvm=cv$cvm, lambda=cv$lambda))
}

OptimLambda <- function(k, ...) {
  # Returns optimal lambda for glmnet.
  #
  # Args:
  #   k: # times to loop through cv.glmnet.
  #   ...: Other args passed to cv.glmnet.
  #
  # Returns:
  #   Lambda associated with minimum average CV error over runs.
  #
  # Example:
  #   OptimLambda(k=100, y=y, x=x, alpha=alpha, nfolds=k)
  #
  require(parallel)
  require(data.table)
  MSEs <- data.table(rbind.fill(mclapply(seq(k), function(dummy) Lambdas(...))))
  return(MSEs[, list(mean.cvm=mean(cvm)), lambda][order(mean.cvm)][1]$lambda)
}

LassoRegularLambda <- OptimLambda(k=100,x=data.matrix(RegularPredictor),y= as.vector(as.matrix(RegularDataResponse)), alpha =1,nfolds = 10, family = "poisson")
LassoPlayoffsLambda <- OptimLambda(k=100,x=data.matrix(PlayoffsPredictor),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =1,nfolds = 10, family = "poisson")
LassoPlayoffsLambda2 <- OptimLambda(k=100,x=data.matrix(PlayoffsPredictor2),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =1,nfolds = 10, family = "poisson")


RidgeRegularLambda <- OptimLambda(k=100,x=data.matrix(RegularPredictor),y= as.vector(as.matrix(RegularDataResponse)), alpha =0,nfolds = 10, family = "poisson")
RidgePlayoffsLambda <- OptimLambda(k=100,x=data.matrix(PlayoffsPredictor),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =0,nfolds = 10, family = "poisson")
RidgePlayoffsLambda2 <- OptimLambda(k=100,x=data.matrix(PlayoffsPredictor2),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =0,nfolds = 10, family = "poisson")


ElasticRegularLambda <- OptimLambda(k=100,x=data.matrix(RegularPredictor),y= as.vector(as.matrix(RegularDataResponse)), alpha =.2607,nfolds = 10, family = "poisson")
ElasticPlayoffsLambda <- OptimLambda(k=100,x=data.matrix(PlayoffsPredictor),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =.0315,nfolds = 10, family = "poisson")
ElasticPlayoffsLambda2 <- OptimLambda(k=100,x=data.matrix(PlayoffsPredictor2),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =.07,nfolds = 10, family = "poisson")

#Alpha = 1 Lasso, Alpha = 0 Ridge, nlambda is an upperbound for the different number of lambdas to be calculated
#Get fits
LassoRegular <- glmnet(x=data.matrix(RegularPredictor),y= as.vector(as.matrix(RegularDataResponse)), alpha =1, lambda = LassoRegularLambda,family = "poisson")
LassoPlayoffs <- glmnet(x=data.matrix(PlayoffsPredictor),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =1, lambda = LassoPlayoffsLambda,family = "poisson")
LassoPlayoffs2 <- glmnet(x=data.matrix(PlayoffsPredictor2),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =1, lambda = LassoPlayoffsLambda2,family = "poisson")
#nlambda is excessively high because the Lasso stops fitting Lambdas automatically once the differences become negligible.
#Will need to choose a reasonable number for the full model
RidgeRegular <- glmnet(x=data.matrix(RegularPredictor),y= as.vector(as.matrix(RegularDataResponse)), alpha =0, lambda = RidgeRegularLambda,family = "poisson")
RidgePlayoffs <- glmnet(x=data.matrix(PlayoffsPredictor),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =0, lambda = RidgePlayoffsLambda,family = "poisson")
RidgePlayoffs2 <- glmnet(x=data.matrix(PlayoffsPredictor2),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =0, lambda = RidgePlayoffsLambda2,family = "poisson")


ElasticRegular <- glmnet(x=data.matrix(RegularPredictor),y= as.vector(as.matrix(RegularDataResponse)), alpha =.2607, lambda = ElasticRegularLambda,family = "poisson")
ElasticPlayoffs <- glmnet(x=data.matrix(PlayoffsPredictor),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =0.0315, lambda = ElasticPlayoffsLambda,family = "poisson")
ElasticPlayoffs2 <- glmnet(x=data.matrix(PlayoffsPredictor2),y= as.vector(as.matrix(PlayoffsDataResponse)), alpha =.07, lambda = ElasticPlayoffsLambda2,family = "poisson")




#Lamba min selects the value of lambda that gives the lowest Cross Validation Error
#Alternatively, Lambda 1se can be used to give a lamba that has error within 1 standard deviation of the min model. This yields a simpler model.

#Make Predictions
LassoRegularPredictions <- predict(LassoRegular, data.matrix(RegularPredictor), LassoRegular$lambda.min)
LassoPlayoffsPredictions <- predict(LassoPlayoffs, data.matrix(PlayoffsPredictor), LassoPlayoffs$lambda.min)
LassoPlayoffsPredictions2 <- predict(LassoPlayoffs2, data.matrix(PlayoffsPredictor2), LassoPlayoffs2$lambda.min)
LassoOOSRegularPredictions <- predict(LassoRegular, data.matrix(OOSRegularPredictor), LassoRegular$lambda.min)
LassoOOSPlayoffsPredictions <- predict(LassoPlayoffs, data.matrix(OOSPlayoffsPredictor), LassoPlayoffs$lambda.min)
LassoOOSPlayoffsPredictions2 <- predict(LassoPlayoffs2, data.matrix(OOSPlayoffsPredictor2), LassoPlayoffs2$lambda.min)



RidgeRegularPredictions <- predict(RidgeRegular, data.matrix(RegularPredictor), RidgeRegular$lambda.min)
RidgePlayoffsPredictions <- predict(RidgePlayoffs, data.matrix(PlayoffsPredictor), RidgePlayoffs$lambda.min)
RidgePlayoffsPredictions2 <- predict(RidgePlayoffs2, data.matrix(PlayoffsPredictor2), RidgePlayoffs2$lambda.min)
RidgeOOSRegularPredictions <- predict(RidgeRegular, data.matrix(OOSRegularPredictor), RidgeRegular$lambda.min)
RidgeOOSPlayoffsPredictions <- predict(RidgePlayoffs, data.matrix(OOSPlayoffsPredictor), RidgePlayoffs$lambda.min)
RidgeOOSPlayoffsPredictions2 <- predict(RidgePlayoffs2, data.matrix(OOSPlayoffsPredictor2), RidgePlayoffs2$lambda.min)
RidgeRecencyPredictions <- predict(RidgePlayoffs2, data.matrix(RecencyPredictor), RidgePlayoffs2$lambda.min)

ElasticRegularPredictions <- predict(ElasticRegular, data.matrix(RegularPredictor), ElasticRegular$lambda.min)
ElasticPlayoffsPredictions <- predict(ElasticPlayoffs, data.matrix(PlayoffsPredictor), ElasticPlayoffs$lambda.min)
ElasticPlayoffsPredictions2 <- predict(ElasticPlayoffs2, data.matrix(PlayoffsPredictor2), ElasticPlayoffs2$lambda.min)
ElasticOOSRegularPredictions <- predict(ElasticRegular, data.matrix(OOSRegularPredictor), ElasticRegular$lambda.min)
ElasticOOSPlayoffsPredictions <- predict(ElasticPlayoffs, data.matrix(OOSPlayoffsPredictor), ElasticPlayoffs$lambda.min)
ElasticOOSPlayoffsPredictions2 <- predict(ElasticPlayoffs2, data.matrix(OOSPlayoffsPredictor2), ElasticPlayoffs2$lambda.min)
ElasticRecencyPredictions <- predict(ElasticPlayoffs2, data.matrix(RecencyPredictor), ElasticPlayoffs2$lambda.min)



#Output Data 
capture.output(coef(LassoRegular, s=LassoRegular$lambda.min), file ="PoissonLassoRegularCoefficients.txt")
capture.output(coef(LassoPlayoffs, s=LassoPlayoffs$lambda.min), file ="PoissonLassoPlayoffsCoefficients.txt")
capture.output(coef(LassoPlayoffs2, s=LassoPlayoffs2$lambda.min), file ="PoissonLassoPlayoffsCoefficients2.txt")



capture.output(coef(RidgeRegular, s=RidgeRegular$lambda.min), file ="PoissonRidgeRegularCoefficients.txt")
capture.output(coef(RidgePlayoffs, s=RidgePlayoffs$lambda.min), file ="PoissonRidgePlayoffsCoefficients.txt")
capture.output(coef(RidgePlayoffs2, s=RidgePlayoffs2$lambda.min), file ="PoissonRidgePlayoffsCoefficients2.txt")

capture.output(coef(ElasticRegular, s=ElasticRegular$lambda.min), file ="PoissonElasticRegularCoefficients.txt")
capture.output(coef(ElasticPlayoffs, s=ElasticPlayoffs$lambda.min), file ="PoissonElasticPlayoffsCoefficients.txt")
capture.output(coef(ElasticPlayoffs2, s=ElasticPlayoffs2$lambda.min), file ="PoissonElasticPlayoffsCoefficients2.txt")


write.csv(data.frame((LassoRegularPredictions)), "PoissonLassoRegularPredictions.csv")
write.csv(data.frame((LassoPlayoffsPredictions)), "PoissonLassoPlayoffsPredictions.csv")
write.csv(data.frame((LassoPlayoffsPredictions2)), "PoissonLassoPlayoffsPredictions2.csv")
write.csv(data.frame((LassoOOSRegularPredictions)), "PoissonLassoOOSRegularPredictions.csv")
write.csv(data.frame((LassoOOSPlayoffsPredictions)), "PoissonLassoOOSPlayoffsPredictions.csv")
write.csv(data.frame((LassoOOSPlayoffsPredictions2)), "PoissonLassoOOSPlayoffsPredictions2.csv")


write.csv(data.frame((RidgeRegularPredictions)), "PoissonRidgeRegularPredictions.csv")
write.csv(data.frame((RidgePlayoffsPredictions)), "PoissonRidgePlayoffsPredictions.csv")
write.csv(data.frame((RidgePlayoffsPredictions2)), "PoissonRidgePlayoffsPredictions2.csv")
write.csv(data.frame((RidgeOOSRegularPredictions)), "PoissonRidgeOOSRegularPredictions.csv")
write.csv(data.frame((RidgeOOSPlayoffsPredictions)), "PoissonRidgeOOSPlayoffsPredictions.csv")
write.csv(data.frame((RidgeOOSPlayoffsPredictions2)), "PoissonRidgeOOSPlayoffsPredictions2.csv")
write.csv(data.frame((RidgeRecencyPredictions)), "PoissonRidgeRecencyPredictions.csv")

write.csv(data.frame((ElasticRegularPredictions)), "PoissonElasticRegularPredictions.csv")
write.csv(data.frame((ElasticPlayoffsPredictions)), "PoissonElasticPlayoffsPredictions.csv")
write.csv(data.frame((ElasticPlayoffsPredictions2)), "PoissonElasticPlayoffsPredictions2.csv")
write.csv(data.frame((ElasticOOSRegularPredictions)), "PoissonElasticOOSRegularPredictions.csv")
write.csv(data.frame((ElasticOOSPlayoffsPredictions)), "PoissonElasticOOSPlayoffsPredictions.csv")
write.csv(data.frame((ElasticOOSPlayoffsPredictions2)), "PoissonElasticOOSPlayoffsPredictions2.csv")
write.csv(data.frame((ElasticRecencyPredictions)), "PoissonElasticRecencyPredictions.csv")
